package EDU.bmrb.net;

import java.lang.*;
import java.io.*;
import java.net.*;

public class BmrbFileGetSend
{
    protected URL cgiURL = null;

    /** Constructor: Creates the URL for the file_get_send CGI program,
      * by using this applets' document base string.  If you call this
      * constructor from within an applet, you can use the applet's
      * <CODE>getDocumentBase()</CODE> method to retrieve this URL.
      * @param docBase the URL of the applet's documentation page in
      *        which this is being run.
      * @exception java.net.MalformedURLException thrown if the URL
      *        docBase string is not valid URL syntax.
      */
    public BmrbFileGetSend( URL docBase )
	throws java.net.MalformedURLException, IOException
    {
        String docBaseString = docBase.toString();
	cgiURL = new URL ( 
		docBaseString.substring( 0, docBaseString.lastIndexOf('/') )
		+ "/file_get_send.cgi" );
    }
    
    /** This method sends the file's contents from a string variable.
      * An example of using it is:
      * <PRE>
      *     bmrbFileGetSend handle = [...snip...];
      *     [...snip...]
      *     handle.sendFile( "filename.txt",
      *                      "This is\nThe file's\nContents.\n" );
      * </PRE>
      * @param remoteFName The name of the file to be stored on the host.
      *                    Be sure to keep it unique.
      * @param fileContents The contents of the file to send to the host.
      * @exception IOException thrown by a number of possible I/O errors.
      * @exception HTTPException if the HTTP response from the server was
      *        anything other than "200 OK", this is thrown and the status
      *        string is in "getMessage()".
      */
    public void sendFile( String remoteFName, byte[] fileContents )
	throws IOException, HTTPException
    {
	int  urlPort = cgiURL.getPort();
        Socket cgiSocket = new Socket( cgiURL.getHost(),
		                       ( urlPort == -1 ) ? 80 : urlPort );
	OutputStream outToCGI = new DataOutputStream(
		cgiSocket.getOutputStream() );
	BufferedReader inFromCGI = new BufferedReader(
		new InputStreamReader( cgiSocket.getInputStream() ) );

	String body = null;
	String boundaryString =
	    "uNiQuE-StRiNg-tHaT-WiLl-lIkElY-NeVeR-ApPeAr-iN-A-FiLe";
	String tcpEndLine = "\r\n";


	// I have no idea what a lot of this means, I am just copying
	// from what I see Netscape doing when I sniff its traffic:
	String header =
	    "POST " + cgiURL.getFile() + " HTTP/1.0" + tcpEndLine +
	    "Connection: Keep-Alive" + tcpEndLine +
	    "User-Agent: bmrbFileGetSend.java code" + tcpEndLine +
	    "Host: " + cgiURL.getHost() + ":" +
	        ( ( urlPort == -1 ) ? 80 : urlPort  ) + tcpEndLine +
	    "Accept: image/jpeg, image/gif, */*" + tcpEndLine +
	    "Accept-Charset: iso-8859-1,*,utf-8" + tcpEndLine +
	    "Content-Type: multipart/form-data; boundary=" + boundaryString
	        + tcpEndLine;

	// Now fill in the form data:
	body = "--" + boundaryString + tcpEndLine + 

	       // This is the "get_send" field:
	       "Content-Disposition: form-data; name=\"get_send\"" +
	           tcpEndLine +
	       tcpEndLine +
	       "send" + tcpEndLine +
	       "--" + boundaryString + tcpEndLine +

	       // This is the "file_name" field:
	       "Content-Disposition: form-data; name=\"file_name\"" +
	           tcpEndLine +
	       tcpEndLine +
	       remoteFName + tcpEndLine +
	       "--" + boundaryString + tcpEndLine +

	       // This is the "file_contents" field:
	       "Content-Disposition: form-data; name=\"file_contents\"" +
		   tcpEndLine +
	       tcpEndLine +
	       new String(fileContents) + tcpEndLine +
	       "--" + boundaryString + "--" + tcpEndLine ;

	// entireContents = header plus body, the whole text to
	// send to the server:
	String entireContents =
	    header +
	    "Content-Length: " + String.valueOf(body.length()) + tcpEndLine +
	    tcpEndLine + // A blank line indicates end of header, start of body:
	    body;

	outToCGI.write( entireContents.getBytes() );
	outToCGI.flush();


	// Done writing request, now it is time to read the response.
	// Let's stick it in a string variable:
	String responseText = inFromCGI.readLine();


	// There's probably more stuff to read, but we don't care
	// about reading it - the status line was enough.
	outToCGI.close();
	inFromCGI.close();

	if( ! responseText.endsWith( "OK" ) )
	{
	    throw new HTTPException( responseText );
	}
    }

    /** This method gets the specified file, and generates a new
      * String containing the contents of the file.
      * <P>
      * @param remoteFName the name of the file to be retrieved from the host.
      * @return A new byte array containing the contents of the file.
      * @exception IOException thrown if there are any I/O errors.
      * @exception HTTPException if the HTTP response from the server was
      *        anything other than "200 OK", this is thrown and the status
      *        string is in "getMessage()".
      */
    public byte[] getFile( String remoteFName )
	throws IOException, HTTPException
    {
	int  urlPort = cgiURL.getPort();
	Socket cgiSocket = new Socket( cgiURL.getHost(),
		                       ( urlPort == -1 ) ? 80 : urlPort );
	OutputStream outToCGI = new DataOutputStream(
		cgiSocket.getOutputStream() );
	BufferedInputStream bufInFromCGI = new BufferedInputStream(
		cgiSocket.getInputStream() );

	String body = null;
	String boundaryString =
	    "uNiQuE-StRiNg-tHaT-WiLl-lIkElY-NeVeR-ApPeAr-iN-A-FiLe";
	String tcpEndLine = "\r\n";


	// I have no idea what a lot of this means, I am just copying
	// from what I see Netscape doing when I sniff its traffic:
	String header =
	    "POST " + cgiURL.getFile() + " HTTP/1.0" + tcpEndLine +
	    "Connection: Keep-Alive" + tcpEndLine +
	    "User-Agent: bmrbFileGetSend.java code" + tcpEndLine +
	    "Host: " + cgiURL.getHost() + ":" +
	        ( ( urlPort == -1 ) ? 80 : urlPort  ) + tcpEndLine +
	    "Accept: image/jpeg, image/gif, */*" + tcpEndLine +
	    "Accept-Charset: iso-8859-1,*,utf-8" + tcpEndLine +
	    "Content-Type: multipart/form-data; boundary=" + boundaryString
	        + tcpEndLine;

	// Now fill in the form data:
	body = "--" + boundaryString + tcpEndLine + 

	       // This is the "get_send" field:
	       "Content-Disposition: form-data; name=\"get_send\"" +
	           tcpEndLine +
	       tcpEndLine +
	       "get" + tcpEndLine +
	       "--" + boundaryString + tcpEndLine +

	       // This is the "file_name" field:
	       "Content-Disposition: form-data; name=\"file_name\"" +
	           tcpEndLine +
	       tcpEndLine +
	       remoteFName + tcpEndLine +
	       "--" + boundaryString + "--" + tcpEndLine ;

	// entireContents = header plus body, the whole text to
	// send to the server:
	String entireContents =
	    header +
	    "Content-Length: " + String.valueOf(body.length()) + tcpEndLine +
	    tcpEndLine + // A blank line indicates end of header, start of body:
	    body;

	outToCGI.write( entireContents.getBytes() );
	outToCGI.flush();

	// Done writing request, now it is time to read the response.
	// Let's stick it in a string variable:

	// Chech the status on the first line: Is the third word "OK"?
	// (Typical response is: "HTTP/1.0 200 OK")
	// -------------------------------------------------------------

	// Read a line in a low-level fashion.  Very slow, but
	// I can't seem to find the right java.io classes to read
	// raw data and filtered text data interchangably.  (What I
	// need is something that has both readLine() *and* a byte-array
	// read method (not char-array).  I could find no such thing, so
	// instead I use a very low-level byte-array reader and I need to
	// do my own input 'cooking' for text lines: )
	int stat = 0;
	byte[] buf = new byte[1];
	String firstLine = "";
	buf[0] = 0;

	while( buf[0] != '\n' && buf[0] >= 0)
	{   stat = bufInFromCGI.read(buf,0,1); // returns -1 on EOF.
	    if( stat >= 1 )
	    {   if( buf[0] != '\r' && buf[0] != '\n')
		{   firstLine += new String(buf);
		}
	    }
	}

	int firstSpacePos = firstLine.indexOf( ' ' );
	int secondSpacePos = firstLine.indexOf( ' ', firstSpacePos+1 );
	int thirdSpacePos = firstLine.indexOf( ' ', secondSpacePos+1 );
	if( thirdSpacePos == -1 )
	    thirdSpacePos = firstLine.length(); // end of string

	String msgDescript = firstLine.substring( secondSpacePos+1, thirdSpacePos );

	if (  !(  msgDescript.equalsIgnoreCase( "OK" ) )  )
	{
	    throw new HTTPException( firstLine );
	}

	// Ignore the rest of the headers, but I have some stub
	// code in here to read the names and values of fields in case
	// I want to ever do something with them later and make this
	// class smarter:
	// --------------------------------------------------------------
	String oneLine = "dummy";
	while( oneLine != null  &&  ! oneLine.equals("") )
	{
	    // Read oneLine, in a low level way:
	    stat = 0;
	    oneLine = "";
	    buf[0] = 0;
	    while( buf[0] != '\n' && buf[0] >= 0)
	    {   stat = bufInFromCGI.read(buf,0,1); // returns -1 on EOF.
		if( stat >= 1 )
		{   if( buf[0] != '\r' && buf[0] != '\n')
		    {   oneLine += new String(buf);
		    }
		}
	    }

	    if(   oneLine == null ||
	          oneLine.equals("") ||
		  oneLine.equals("\r") ||
		  oneLine.equals("\n") ||
		  oneLine.equals("\r\n")    )
	    {
		oneLine = null;
	    }
	    else
	    {
		String headerFieldName =
			oneLine.substring( 0, oneLine.indexOf( ':' ) );
		String headerFieldValue = 
			oneLine.substring( oneLine.indexOf( ':' ) + 2,
			oneLine.length() );
	    }
	}


	// Everything else after the header is the body - stuff it
	// in the buffer.
	// --------------------------------------------------------
	byte[] cbuf = new byte[2048];
	byte[] retVal = new byte[0];
	int  bytesSoFar = 0;
	int  result = 0;
	int  i,j;
	while( result >= 0)
        {
	    result = bufInFromCGI.read( cbuf, 
		                         0,
					 2048 );
	    // Append result onto the byte array:
	    // Java can't 'realloc()', so instead we make a new
	    // byte array and populate it with the old one's contents:
	    if( result >= 0 )
	    {
		bytesSoFar += result;
		byte[] newArr = new byte[ bytesSoFar ];
		for( i = 0 ; i < retVal.length ; i++ )
		    newArr[i] = retVal[i];
		for( j = 0 ; j < result ; j++ )
		    newArr[i+j] = cbuf[j];
		retVal = newArr;
		newArr = null;
	    }
	}

	outToCGI.close();
	bufInFromCGI.close();

	return retVal;
    }

    /** This method deletes the specified file from the host
      * @param remoteFName the name of the file to be removed from the host.
      * @exception IOExcpetion when some I/O error happened in the traffic.
      * @exception HTTPException when the "HTTP/1.0 XXX ..." string returned
      * indicates some error.  The getMessage() field of the exception
      * contains the errror message returned.
      */
    public void delFile( String remoteFName )
	throws IOException, HTTPException
    {
	int  urlPort = cgiURL.getPort();
	Socket cgiSocket = new Socket( cgiURL.getHost(),
		                       ( urlPort == -1 ) ? 80 : urlPort );
	OutputStream outToCGI = new DataOutputStream(
		cgiSocket.getOutputStream() );
	BufferedInputStream bufInFromCGI = new BufferedInputStream(
		cgiSocket.getInputStream() );

	String body = null;
	String boundaryString =
	    "uNiQuE-StRiNg-tHaT-WiLl-lIkElY-NeVeR-ApPeAr-iN-A-FiLe";
	String tcpEndLine = "\r\n";


	// I have no idea what a lot of this means, I am just copying
	// from what I see Netscape doing when I sniff its traffic:
	String header =
	    "POST " + cgiURL.getFile() + " HTTP/1.0" + tcpEndLine +
	    "Connection: Keep-Alive" + tcpEndLine +
	    "User-Agent: bmrbFileGetSend.java code" + tcpEndLine +
	    "Host: " + cgiURL.getHost() + ":" + 
	        ( ( urlPort == -1 ) ? 80 : urlPort ) + tcpEndLine +
	    "Accept: image/jpeg, image/gif, */*" + tcpEndLine +
	    "Accept-Charset: iso-8859-1,*,utf-8" + tcpEndLine +
	    "Content-Type: multipart/form-data; boundary=" + boundaryString
	        + tcpEndLine;

	// Now fill in the form data:
	body = "--" + boundaryString + tcpEndLine + 

	       // This is the "get_send" field:
	       "Content-Disposition: form-data; name=\"get_send\"" +
	           tcpEndLine +
	       tcpEndLine +
	       "del" + tcpEndLine +
	       "--" + boundaryString + tcpEndLine +

	       // This is the "file_name" field:
	       "Content-Disposition: form-data; name=\"file_name\"" +
	           tcpEndLine +
	       tcpEndLine +
	       remoteFName + tcpEndLine +
	       "--" + boundaryString + "--" + tcpEndLine ;

	// entireContents = header plus body, the whole text to
	// send to the server:
	String entireContents =
	    header +
	    "Content-Length: " + String.valueOf(body.length()) + tcpEndLine +
	    tcpEndLine + // A blank line indicates end of header, start of body:
	    body;

	outToCGI.write( entireContents.getBytes() );
	outToCGI.flush();

	// Done writing request, now it is time to read the response.
	// Let's stick it in a string variable:

	// Chech the status on the first line: Is the third word "OK"?
	// (Typical response is: "HTTP/1.0 200 OK")
	// -------------------------------------------------------------

	// Read a line in a low-level fashion.  Very slow, but
	// I can't seem to find the right java.io classes to read
	// raw data and filtered text data interchangably.  (What I
	// need is something that has both readLine() *and* a byte-array
	// read method (not char-array).  I could find no such thing, so
	// instead I use a very low-level byte-array reader and I need to
	// do my own input 'cooking' for text lines: )
	int stat = 0;
	byte[] buf = new byte[1];
	String firstLine = "";
	buf[0] = 0;

	while( buf[0] != '\n' && buf[0] >= 0)
	{   stat = bufInFromCGI.read(buf,0,1); // returns -1 on EOF.
	    if( stat >= 1 )
	    {   if( buf[0] != '\r' && buf[0] != '\n')
		{   firstLine += new String(buf);
		}
	    }
	}

	int firstSpacePos = firstLine.indexOf( ' ' );
	int secondSpacePos = firstLine.indexOf( ' ', firstSpacePos+1 );
	int thirdSpacePos = firstLine.indexOf( ' ', secondSpacePos+1 );
	if( thirdSpacePos == -1 )
	    thirdSpacePos = firstLine.length(); // end of string

	String msgDescript = firstLine.substring( secondSpacePos+1, thirdSpacePos );

	if (  !(  msgDescript.equalsIgnoreCase( "OK" ) )  )
	{
	    throw new HTTPException( firstLine );
	}

	// Ignore the rest of the headers, but I have some stub
	// code in here to read the names and values of fields in case
	// I want to ever do something with them later and make this
	// class smarter:
	// --------------------------------------------------------------
	String oneLine = "dummy";
	while( oneLine != null  &&  ! oneLine.equals("") )
	{
	    // Read oneLine, in a low level way:
	    stat = 0;
	    oneLine = "";
	    buf[0] = 0;
	    while( buf[0] != '\n' && buf[0] >= 0)
	    {   stat = bufInFromCGI.read(buf,0,1); // returns -1 on EOF.
		if( stat >= 1 )
		{   if( buf[0] != '\r' && buf[0] != '\n')
		    {   oneLine += new String(buf);
		    }
		}
	    }

	    if(   oneLine == null ||
	          oneLine.equals("") ||
		  oneLine.equals("\r") ||
		  oneLine.equals("\n") ||
		  oneLine.equals("\r\n")    )
	    {
		oneLine = null;
	    }
	    else
	    {
		String headerFieldName =
			oneLine.substring( 0, oneLine.indexOf( ':' ) );
		String headerFieldValue = 
			oneLine.substring( oneLine.indexOf( ':' ) + 2,
			oneLine.length() );
	    }
	}

	outToCGI.close();
	bufInFromCGI.close();
    }
}
