# - run this as a perl script.

# This fixes a (IMO) mistake in javacc whereby it holds a big
# linked list of tokens such that they never get garbage
# collected.  This changes the javacc-generated code so that
# it cuts off the links and allows garbage collection to clean
# up.  If this is not done, the generated parser runs out of
# memory on large files (takes > 100 Mb to parse a 3 Mb file).
#
# Unfortunately, this 'bug' is in the code generated by javacc,
# so we have to clean up the code 'after the fact' in the generated
# StarParser.java file, by inserting a line.  That's what this
# cheesy little perl script is for.  It finds the appropriate place
# in the outputted Java file and adds the nessecery line of code.
# This was done programmatically so that I could continue to edit the
# javacc code generator and edit star.javacc as my starting source file.
#
# OBVIOUSLY, THIS MIGHT BREAK WHEN A MORE RECENT VERSION OF
# JavaCC IS USED, IF IT HAPPENS	TO CHANGE HOW IT MAKES THIS
# ONE FUNCTION LOOK.

print STDERR "Fixing JavaCC Heap Abuse: ";

# This operates on STDIN/STDOUT
$inside_right_function = 0;
$inside_right_if_block = 0;
while( $line = <STDIN> )
{

    # Check to see if the right function header for the part
    # of the code we care about was found:
    if( $line =~ /Token\s*jj_consume_token\s*(.*)/ )
    {
	$inside_right_function = 1;
        print STDERR "(found jj_consume_token) " ;
    }

    # Check for the return statement, insert our stuff before it:
    if(     $inside_right_function &&
	    ($line =~ /return\s*token/  )   )
    {
	print STDERR "(inserting line of code) " ;
	print "\n" .
	      "      // The next lines inserted by BMRB into the \n" .
	      "      // JavaCC-generated code.  This fixes a garbage \n" .
	      "      // collection problem that was making us run out \n" .
	      "      // of memory parsing large files.  Without the \n" .
	      "      // next line, the default JavaCC-generated file \n" .
	      "      // would have left a huge linked-list laying \n" .
	      "      // around of all the tokens in a rule.  This is \n" .
	      "      // devastating when dealing with big loops. \n" .
	      "\n" .
	      "      // { BEGIN section inserted by BMRB:\n" .
	      "           oldToken.next = null ;\n" .
	      "           oldToken.specialToken = null ;\n" .
	      "      // } END section inserted by BMRB:\n" .
	      "\n" ;

	$inside_right_function = 0; # don't do it again now that it was
	                            # done once.
    }

    print $line; # echo everything to output
}

print STDERR "\n";
