package EDU.bmrb.xeasy;

import java.io.*;
import java.util.*;
import EDU.bmrb.starlibj.*;

//java nmr_to_xeasy infile xeasyfile mapfile
class nmr_to_xeasy{
  public static void main(String argv[]){
    if(argv.length != 2 ){
	  System.out.println("Usage:java nmr_to_xeasy infile outfile");
	  System.exit(1);
	}
  

    String in_file = argv[0];
    String out_file = argv[1];
  
    //If the input file is not in NMR-STAR file and just a loop, make
    //it to NMR-STAR format.
    boolean is_nmr = is_NMR(in_file);

    if(is_nmr)
      System.out.println(in_file+" is a NMR-STAR file.");
    else
      System.out.println(in_file+" is not a NMR-STAR file.");
    
    StarParser   myParser = null;
    StarFileNode sfn = null;
    
      
    PrintWriter xeasyfile = null;
    PrintWriter mapfile   = null;
  
    try
    {
	   xeasyfile = new PrintWriter( new FileOutputStream(in_file+".xeasy") );
           mapfile   = new PrintWriter( new FileOutputStream(in_file+".map") );
           
	   myParser = new StarParser( new java.io.FileInputStream(in_file) );
           myParser.StarFileNodeParse( myParser);
           sfn = (StarFileNode)myParser.endResult();
	   
	   convert(sfn, xeasyfile, mapfile);
  
           
     } 
     catch(IOException e)
     {
          System.out.println("IOException:"+e.getMessage() );
          e.printStackTrace();
     }
     catch(EDU.bmrb.starlibj.ParseException e) {
          System.out.println("Parse Exception: " + e.getMessage() );
          e.printStackTrace();
     }

    xeasyfile.close();
    mapfile.close();
  }

  
  static void convert(StarFileNode fileroot, PrintWriter xeasyfile, PrintWriter mapfile){
     VectorCheckType saveFrameHits = null;
     VectorCheckType matches = null;
     LoopTableNode table = null;
     DataLoopNameListNode orig_names = null;
     int rowCount = 0;
     int colCount = 0;

     //Track the position of eaxh tag in the loop.
     int shift_id_pos = 0;
     int residue_seq_code_pos = 0;
     int residue_label_pos = 0;
     int atom_name_pos = 0;
     int atom_type_pos = 0;
     int shift_value_pos = 0;
     int shift_error_pos = 0;
     int ambiguity_code_pos = 0;

     //Track the value of each tag.
     String shift_id = null;
     String residue_seq_code = null;
     String residue_label = null;
     String atom_name = null;
     String atom_type = null;
     String shift_value = null;
     String shift_error = null;
     String ambiguity_code = null;
     String old_seq_code = new String("-1");
     //Find the saveframendoe with a tag named as "_Saveframe_category" and with
     //value of "assigned_chemical_shifts".

     try{
        saveFrameHits = fileroot.searchForTypeByTagValue(Class.forName(StarValidity.pkgName()+".SaveFrameNode"),
	     "_Saveframe_category", "assigned_chemical_shifts");
     }
     catch( ClassNotFoundException exc ){
        System.err.println( "Should never happen exception: " +
                                exc.getMessage() );
        exc.printStackTrace();
     }


     System.out.println("saveFrameHits.size()="+saveFrameHits.size()); //Delete me 
     if(saveFrameHits.size() != 1){
	 System.out.println("ERROR: not only one save frame with" +
		 "tag _Savefram_category assigned_chemical_shifts");
     }else{
	 try{
	    matches =((SaveFrameNode)(saveFrameHits.elementAt(0))).searchForTypeByName(Class.forName(StarValidity.pkgName()+".DataLoopNode"),
		 "_Atom_shift_assign_ID");
	 }
	 catch( ClassNotFoundException exc ){
            System.err.println( "Should never happen exception: " +
                                exc.getMessage() );
            exc.printStackTrace();
         }

	 if(matches.size() != 1){
	     System.out.println("matches size was " + matches.size());
	    System.out.println("ERROR: not only one dataloopnode with tag "+
		    "_Atom_shift_assign_ID");
	 }else{
	     table = ((DataLoopNode) matches.elementAt(0)).getVals();
	     orig_names = ((DataLoopNode)matches.elementAt(0)).getNames();
	     LoopNameListNode namelist = ((LoopNameListNode)orig_names.elementAt(0));
	     rowCount = table.size();
	     colCount = namelist.size();

	     for(int i=0; i<colCount; i++){
	        String tmp = ((DataNameNode)namelist.elementAt(i)).getValue();
		if( tmp.equalsIgnoreCase("_Atom_shift_assign_ID") ){
                    shift_id_pos = i;
		    continue;
		}
                if( tmp.equalsIgnoreCase("_Residue_seq_code") ){
                    residue_seq_code_pos = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Residue_label") ){
                    residue_label_pos = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Atom_name") ){
                    atom_name_pos = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Atom_type") ){
                    atom_type_pos = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Chem_shift_value") ){
                    shift_value_pos  = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Chem_shift_value_error") ){
                    shift_error_pos = i;
		    continue;
		}
		if( tmp.equalsIgnoreCase("_Chem_shift_ambiguity_code") ){
                    ambiguity_code_pos = i;
		    continue;
		}
	     }

	     //Go through the loop table row by row to convert NMR-STAR to XEASY
	     LoopRowNode tmpRow;
	     for(int curRow = 0; curRow<rowCount; curRow++){
 		tmpRow = (LoopRowNode)table.elementAt(curRow);
		shift_id         = ((DataValueNode)tmpRow.elementAt(shift_id_pos)).getValue();
		residue_seq_code = ((DataValueNode)tmpRow.elementAt(residue_seq_code_pos)).getValue();
                residue_label    = ((DataValueNode)tmpRow.elementAt( residue_label_pos)).getValue();
                atom_name        = ((DataValueNode)tmpRow.elementAt(atom_name_pos)).getValue();
                atom_type        = ((DataValueNode)tmpRow.elementAt(atom_type_pos)).getValue();
                shift_value      = ((DataValueNode)tmpRow.elementAt(shift_value_pos)).getValue();
                shift_error      = ((DataValueNode)tmpRow.elementAt(shift_error_pos)).getValue();
	        ambiguity_code   = ((DataValueNode)tmpRow.elementAt(ambiguity_code_pos)).getValue();			 
	     }

	     //Output to xeasy file
             xeasyfile.print(shift_id+"\t"+shift_value+"\t");
	     if(shift_error.equals(".")){
                xeasyfile.print("0.000"+"\t");
	     }else{
 		xeasyfile.print(shift_value+"\t");
	     }
	     xeasyfile.print(atom_name+"\t"+residue_seq_code+"\n");
		     
	     //Output the mapping file 
	     if( old_seq_code.equals(residue_seq_code)){
                  //Do nothing
	     }else{
	          mapfile.print(residue_seq_code+"\n");
		  old_seq_code = residue_seq_code;
	     }
	 }
     }   
  }

  static boolean is_NMR(String filename){
      try{
        BufferedReader in =
          new BufferedReader(new FileReader(filename));
        for(;;){
          String line = in.readLine();
          if(line == null)  break;
         
          StringTokenizer tokens = new StringTokenizer(line);
          String next_token = tokens.nextToken(" ");
          if( next_token.startsWith("data_")) return true;
          else break;
        }
      
        PrintWriter outFile = new PrintWriter( new FileOutputStream(filename+".out"));
        in = new BufferedReader(new FileReader(filename));
        //Change the file into NMR-STAR fomat.
        outFile.println("data_0000");
        for(;;){
          String line = in.readLine();
          if(line == null) break;
          outFile.println(line);
        }
        outFile.close();
      }catch(IOException e){
           System.err.println("IOException is caughted.");
      }
      return false;
    }

}
